package com.nisum.poc.facade.service.impl;

import com.nisum.poc.facade.client.IDepartmentClient;
import com.nisum.poc.facade.client.IEmployeeClient;
import com.nisum.poc.facade.client.IProjectClient;
import com.nisum.poc.facade.response.DepartmentDetails;
import com.nisum.poc.facade.response.EmployeeDetails;
import com.nisum.poc.facade.response.EmployeeFacadeResponse;
import com.nisum.poc.facade.response.ProjectDetails;
import com.nisum.poc.facade.service.IEmployeeService;
import org.modelmapper.ModelMapper;
import org.modelmapper.convention.MatchingStrategies;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class EmployeeService implements IEmployeeService {

    private final IEmployeeClient employeeClient;

    private final IDepartmentClient departmentClient;

    private final IProjectClient projectClient;

    @Autowired
    public EmployeeService (IEmployeeClient employeeClient, IDepartmentClient departmentClient, IProjectClient projectClient){
        this.employeeClient = employeeClient;
        this.departmentClient = departmentClient;
        this.projectClient = projectClient;

    }
    @Override
    public Flux<EmployeeDetails> fetchEmployeeDetails() {
        return employeeClient.fetchAllEmployee();
    }

    @Override
    public Mono<EmployeeFacadeResponse> fetchEmployeeById(Long empId) {


        Mono<EmployeeDetails> monoEmployeeDetails = employeeClient.findEmployeeDetails(empId);
        return monoEmployeeDetails.map(this ::findProjectAndDeptDetails);

    }

    private EmployeeFacadeResponse findProjectAndDeptDetails(EmployeeDetails employeeDetails){

        EmployeeFacadeResponse response = constructEmployeeFacadeResponse(employeeDetails);

        Long deptId = employeeDetails.getDeptId();
        Long projectId = employeeDetails.getProjectId();
        departmentClient.fetchDepartmentById(deptId).map(dept -> getFacadeResponse(dept, response));

        projectClient.findProjectDetails(projectId).map(proj -> getFacadeResponse(proj, response));

        return response;
    }

    private Object getFacadeResponse(ProjectDetails project, EmployeeFacadeResponse response) {
        response.setProjectDetails(project);
        return response;
    }


    private Object getFacadeResponse(DepartmentDetails dept, EmployeeFacadeResponse response) {
        response.setDepartmentDetails(dept);
        return response;
    }
    private EmployeeFacadeResponse constructEmployeeFacadeResponse(EmployeeDetails employeeDetails) {

        EmployeeFacadeResponse employeeFacadeResponse;

        ModelMapper modelMapper = new ModelMapper();
        modelMapper.getConfiguration().setMatchingStrategy(MatchingStrategies.STRICT);
        employeeFacadeResponse = modelMapper.map(employeeDetails, EmployeeFacadeResponse.class);

        return employeeFacadeResponse;
    }
}
